/* $USAGI: ipv6only_tinytest.c,v 1.2 2001/01/25 09:37:35 yoshfuji Exp $ */

/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#ifndef IPV6_V6ONLY
#define IPV6_V6ONLY	26	/*XXX*/
#endif

int main(int argc, char **argv){
	struct addrinfo hints, *res0, *res;
	int s;
#ifdef STREAM
	int t;
#endif
	int gai;
	char addrbuf[128];
	int on;
	struct sockaddr_storage sa;
	int salen;
	char *serv;
	char buf[128];

	if (argc < 2) {
		printf ("%s port\n", argv[0]);
		exit(1);
	}

	serv = argv[1];

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_INET6;
#ifdef STREAM
	hints.ai_socktype = SOCK_STREAM;
#else
	hints.ai_socktype = SOCK_DGRAM;
#endif
	hints.ai_flags = AI_PASSIVE;
	gai = getaddrinfo(NULL, serv, &hints, &res0);
	if (gai) {
		printf("getaddrinfo(): %s\n", gai_strerror(gai));
		exit(1);
	}
	s = socket(res0->ai_family, res0->ai_socktype, res0->ai_protocol);
	if (s < 0){
		perror("socket");
		exit(1);
	}
	on = 1;
	if (setsockopt(s, SOL_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0) {
		perror("setsockopt");
		exit(1);
	}
	if (bind(s, res0->ai_addr, res0->ai_addrlen) < 0) {
		perror("bind");
		exit(1);
	}
#ifdef STREAM
	listen(s, 5);
	salen = sizeof(sa);
	if ((t = accept(s, (struct sockaddr *)&sa, &salen)) < 0){
		perror("accept");
		close(s);
		exit(1);
	}
#else
	if (recvfrom(s, buf, sizeof(buf), 0,
			(struct sockaddr *)&sa, &salen) < 0){
		perror("recvfrom");
		close(s);
		exit(1);
	}
#endif
	if (getnameinfo((struct sockaddr *)&sa, salen, 
			addrbuf, sizeof(addrbuf), NULL, 0,
			NI_NUMERICHOST)) {
		printf("getnameinfo()\n");
		close(s);
#ifdef STREAM
		close(t);
#endif
		exit(1);
	}
	printf("from %s\n", addrbuf);
	close(s);
#ifdef STREAM
	close(t);
#endif
	exit(0);
}

